@@ -6,3 +6,7 @@ Rv = 461.50 # [J/kg/K]
 kappa = 2./7. # []
 cp = Rd/kappa # [J/kg/K]
 sigma = 5.6734e-8 # [W/m2/K4]
+ES0 = 610.78 # [Pa]
+HLV = 2.5e6 # [J/kg]
+Tfreeze = 273.16 # [K]
+p0 = 1e3 # [hPa]
